searchNodes=[{"doc":"A custom Elixir compiler task that checks for cross-references between specific modules, known as &quot;contexts&quot;.","ref":"Mix.Tasks.Compile.Contexted.html","title":"Mix.Tasks.Compile.Contexted","type":"task"},{"doc":"Sets the custom compiler tracer to the Contexted.Tracer module.","ref":"Mix.Tasks.Compile.Contexted.html#run/1","title":"Mix.Tasks.Compile.Contexted.run/1","type":"function"},{"doc":"Contexted helps you structure Phoenix contexts by enforcing separation, allowing modularization, and generating common CRUD operations. Features Contexted.Tracer - trace and enforce definite separation between specific context modules. Contexted.Delegator - divide the big context module into smaller parts and use delegations to build the final context. Contexted.CRUD - auto-generate the most common CRUD operations whenever needed. Installation Add the following to your mix.exs file: defp deps do [ { :contexted , &quot;~&gt; 0.3.4&quot; } ] end Then run mix deps.get . Step by step overview To describe a sample usage of this library, let's assume that your project has three contexts: Account Subscription Blog Our goal, as the project grows, is to: Keep contexts separate and not create any cross-references. For this to work, we'll raise errors during compilation whenever such a cross-reference happens. Divide each context into smaller parts so that it is easier to maintain. In this case, we'll refer to each of these parts as Subcontext . It's not a new term added to the Phoenix framework but rather a term proposed to emphasize that it's a subset of Context. For this to work, we'll use delegates. Not repeat ourselves with common business logic operations. For this to work, we'll be using CRUD functions generator, since these are the most common. Keep contexts separate It's very easy to monitor cross-references between context modules with the contexted library. First, add contexted as one of the compilers in mix.exs : def project do [ ... compilers : [ :contexted ] ++ Mix . compilers ( ) , ... ] end Next, define a list of contexts available in the app inside config file: config :contexted , contexts : [ # list of context modules goes here, for instance: # [App.Account, App.Subscription, App.Blog] ] And that's it. From now on, whenever you will cross-reference one context with another, you will see an error raised during compilation. Here is an example of such an error: == Compilation error in file lib / app / accounts . ex == ** (RuntimeError) You can&#39;t reference App.Blog context within App.Accounts context. See more in Contexted.Tracer . Exclude files and folders from cross-references context check In special cases, you may need to exclude certain folders or files from cross-reference checks due to project structure or naming conventions. To do this, add a list of exclusions in config exclude_paths option: config :contexted , exclude_paths : [ &quot;app/test&quot; ] Dividing each context into smaller parts To divide big Context into smaller Subcontexts, we can use delegate_all/1 macro from Contexted.Delegator module. Let's assume that the Account context has User , UserToken and Admin resources. Here is how we can split the context module: # Users subcontext defmodule App.Account.Users do def get_user ( id ) do ... end end # UserTokens subcontext defmodule App.Account.UserTokens do def get_user_token ( id ) do ... end end # Admins subcontext defmodule App.Account.Admins do def get_admin ( id ) do ... end end # Account context defmodule App.Account do import Contexted.Delegator delegate_all App.Account.Users delegate_all App.Account.UserTokens delegate_all App.Account.Admins end From now on, you can treat the Account context module as the API for the &quot;outside&quot; world. Instead of calling: App.Account.Users . find_user ( 1 ) You will simply do: App.Account . find_user ( 1 ) Being able to access docs and specs in auto-delegated functions Both docs and specs are attached as metadata of module once it's compiled and saved as .beam . In reference to the example of App.Account context, it's possible that App.Account.Users will not be saved in .beam file before the delegate_all macro is executed. Therefore, first, all of the modules have to be compiled, and saved to .beam and only then we can create @doc and @spec of each delegated function. As a workaround, in Contexted.Tracer.after_compiler/1 all of the contexts .beam files are first deleted and then recompiled. This is an opt-in functionality, as it extends compilation time. If you want to enable it, set the following config values: config :contexted , app : :your_app_name , # replace &#39;your_app_name&#39; with your real app name enable_recompilation : true You may also want to enable it only for certain environments, like dev . Please also note that when this functionality is enabled, during the recompilation process, warnings are temporarily silenced to avoid logging conflict warnings. It will still log warnings as intended, during the first compilation, therefore it won't have any affect on normal compilation flow. See more in Contexted.Delegator . Don't repeat yourself with CRUD operations In most web apps CRUD operations are very common. Most of these, have the same pattern. Why not autogenerate them? Here is how you can generate common CRUD operations for App.Account.Users : defmodule App.Account.Users do use Contexted.CRUD , repo : App.Repo , schema : App.Accounts.User end This will generate the following functions: [ change_user : 1 , change_user : 2 , create_user : 0 , create_user : 1 , create_user! : 0 , create_user! : 1 , delete_user : 1 , delete_user! : 1 , get_user : 1 , get_user! : 1 , list_users : 0 , update_user : 1 , update_user : 2 , update_user! : 1 , update_user! : 2 ] See more in Contexted.CRUD .","ref":"Contexted.html","title":"Contexted","type":"module"},{"doc":"The Contexted.CRUD module generates common CRUD (Create, Read, Update, Delete) functions for a context, similar to what mix phx gen context task generates. Options :repo - The Ecto repository module used for database operations (required) :schema - The Ecto schema module representing the resource that these CRUD operations will be generated for (required) :exclude - A list of atoms representing the functions to be excluded from generation (optional) :plural_resource_name - A custom plural version of the resource name to be used in function names (optional). If not provided, singular version with 's' ending will be used to generate list function Usage defmodule MyApp.Accounts do use Contexted.CRUD , repo : MyApp.Repo , schema : MyApp.Accounts.User , exclude : [ :delete ] , plural_resource_name : &quot;users&quot; end This sample usage will generate all CRUD functions for MyApp.Accounts.User resource, excluding delete_user/1 . Generated Functions The following functions are generated by default. Any of them can be excluded by adding their correspoding atom to the :exclude option. list_{plural resource name} - Lists all resources in the schema. get_{resource name} - Retrieves a resource by its ID. Returns nil if not found. get_{resource name}! - Retrieves a resource by its ID. Raises an error if not found. create_{resource name} - Creates a new resource with the provided attributes. Returns an :ok tuple with the resource or an :error tuple with changeset. create_{resource name}! - Creates a new resource with the provided attributes. Raises an error if creation fails. update_{resource name} - Updates an existing resource with the provided attributes. Returns an :ok tuple with the resource or an :error tuple with changeset. update_{resource name}! - Updates an existing resource with the provided attributes. Raises an error if update fails. delete_{resource name} - Deletes an existing resource. Returns an :ok tuple with the resource or an :error tuple with changeset. delete_{resource name}! - Deletes an existing resource. Raises an error if delete fails. change_{resource name} - Returns changeset for given resource. {resource name} and {plural resource name} will be replaced by the singular and plural forms of the resource name.","ref":"Contexted.CRUD.html","title":"Contexted.CRUD","type":"module"},{"doc":"The Contexted.Delegator module provides a macro to delegate all functions defined within a specific module. This module can be used to forward all function calls from one module to another without the need to write individual defdelegate statements for each function. Usage To use the delegate_all macro, simply include it in your module and pass the target module as an argument: defmodule MyContextModule do import Contexted.Delegator delegate_all MyTargetSubcontextModule end All public functions defined in MyTargetSubcontextModule will now be accessible within MyContextModule . Note: This macro should be used with caution, as it may lead to unexpected behaviors if two modules with overlapping function names are delegated. Additional configuration The Contexted.Delegator module can be used without Mix.Tasks.Compile.Contexted as one of the compilers. However, if you wish to enable automatic @doc and @spec generation for delegated functions, you will need to set the following config: config :contexted , enable_recompilation : true","ref":"Contexted.Delegator.html","title":"Contexted.Delegator","type":"module"},{"doc":"Delegates all public functions of the given module. Examples defmodule MyContextModule do import Contexted.Delegator delegate_all MyTargetSubcontextModule end All public functions defined in MyTargetSubcontextModule will now be accessible within MyContextModule .","ref":"Contexted.Delegator.html#delegate_all/1","title":"Contexted.Delegator.delegate_all/1","type":"macro"},{"doc":"The Contexted.Tracer module provides a set of functions to trace and enforce separation between specific modules, known as &quot;contexts&quot;. This is useful when you want to ensure that certain modules do not reference each other directly within your application.","ref":"Contexted.Tracer.html","title":"Contexted.Tracer","type":"module"},{"doc":"To support automatic docs and specs generation in delegated functions inside contexts, all of the modules have to be compiled first. Because of that, we need to run this operation in after compiler callback in two steps: Remove all contexts beam files. Generate contexts beam files again. This is an opt-in step, so in order to enable it, you will have to set this config: config :contexted , enable_recompilation : true","ref":"Contexted.Tracer.html#after_compiler/1","title":"Contexted.Tracer.after_compiler/1","type":"function"},{"doc":"","ref":"Contexted.Tracer.html#get_beam_files_folder_path/0","title":"Contexted.Tracer.get_beam_files_folder_path/0","type":"function"},{"doc":"Trace events are emitted during compilation. trace function verifies if the provided event contains cross-references between two contexts. If so, raises an error with an appropriate message.","ref":"Contexted.Tracer.html#trace/2","title":"Contexted.Tracer.trace/2","type":"function"},{"doc":"The Contexted.ModuleAnalyzer defines utils functions that analyze and extract information from other modules.","ref":"Contexted.ModuleAnalyzer.html","title":"Contexted.ModuleAnalyzer","type":"module"},{"doc":"Generates a list of unique argument names based on the given arity.","ref":"Contexted.ModuleAnalyzer.html#generate_random_function_arguments/1","title":"Contexted.ModuleAnalyzer.generate_random_function_arguments/1","type":"function"},{"doc":"Finds and returns the @doc definition in string format for the specified function name and arity. Returns nil if the function is not found in the function docs.","ref":"Contexted.ModuleAnalyzer.html#get_function_doc/3","title":"Contexted.ModuleAnalyzer.get_function_doc/3","type":"function"},{"doc":"Finds and returns the @spec definition in string format for the specified function name and arity. Returns nil if the function is not found in the specs.","ref":"Contexted.ModuleAnalyzer.html#get_function_spec/3","title":"Contexted.ModuleAnalyzer.get_function_spec/3","type":"function"},{"doc":"Fetches the @doc definitions for all functions within the given module.","ref":"Contexted.ModuleAnalyzer.html#get_functions_docs/1","title":"Contexted.ModuleAnalyzer.get_functions_docs/1","type":"function"},{"doc":"Fetches the @spec definitions for all functions within the given module.","ref":"Contexted.ModuleAnalyzer.html#get_functions_specs/1","title":"Contexted.ModuleAnalyzer.get_functions_specs/1","type":"function"},{"doc":"The Contexted.Utils defines utils functions for other modules.","ref":"Contexted.Utils.html","title":"Contexted.Utils","type":"module"},{"doc":"","ref":"Contexted.Utils.html#get_config_app/0","title":"Contexted.Utils.get_config_app/0","type":"function"},{"doc":"Returns contexts option value from contexted config or [] if it's not set.","ref":"Contexted.Utils.html#get_config_contexts/0","title":"Contexted.Utils.get_config_contexts/0","type":"function"},{"doc":"Returns exclude_paths option value from contexted config or [] if it's not set.","ref":"Contexted.Utils.html#get_config_exclude_paths/0","title":"Contexted.Utils.get_config_exclude_paths/0","type":"function"},{"doc":"Checks is enable_recompilation option is set.","ref":"Contexted.Utils.html#recompilation_enabled?/0","title":"Contexted.Utils.recompilation_enabled?/0","type":"function"}]